#This script shows how to prepare the data for the analyses presented in the paper 
#"Decomposing the True Score Variance in Rated Responses to Divergent Thinking-Tasks for Assessing Creativity:
# A Multitrait-Multimethod Analysis".


#First, download the data from: https://osf.io/a9qnc 
#Unpack the ZIP-file and save the rda-files
#"cleverness_ratings.rda" and "creative_quality_ratings.rda"
#in your current R-working directory.

#Now, load the objects from the rda-files:
load("cleverness_ratings.rda")
load("creative_quality_ratings.rda")

#Define a function that calculates the .75-quantile of a variable
#and removes missing values before doing so:
quantileuse <- function(x){
  quantile(x, probs = .75, na.rm=T)
}

#For raters 6, 4, and 7 use the .75-quantile function from above 
#to compute the .75-quantile for the individual-idea creative quality-ratings of a set
#of produced ideas for every target (subject_id) and object (AUT) combination.
#Save these as separate data-frames
#(translation of object-names: Büroklammer = paperclip, Mülltüte = garbage-bag, Seil = rope):
cqr6 <- as.data.frame(aggregate(creativeq.rater6 ~ subject_id + object, data = cqd, FUN=quantileuse))
cqr4 <- as.data.frame(aggregate(creativeq.rater4 ~ subject_id + object, data = cqd, FUN=quantileuse))
cqr7 <- as.data.frame(aggregate(creativeq.rater7 ~ subject_id + object, data = cqd, FUN=quantileuse))

#reshape these data into wide format (separate columns for each object):
cqr4w <- reshape(cqr4, idvar = "subject_id", timevar = "object", direction = "wide")
cqr6w <- reshape(cqr6, idvar = "subject_id", timevar = "object", direction = "wide")
cqr7w <- reshape(cqr7, idvar = "subject_id", timevar = "object", direction = "wide")

#Create an object "cledat2" that contains the same data as "cledat" (the cleverness-ratings)
#but without the cleverness-means across raters:
cledat2 <- cledat[,-c(7,13,19)]

#Merge the creative quality and cleverness ratings step by step:
m1 <- merge(cledat2, cqr4w, by = "subject_id")
m1 <- merge(m1, cqr6w, by = "subject_id")
m1 <- merge(m1, cqr7w, by = "subject_id")

#Rename the column-names of the merged data (m1) in a more convenient way:
#id = target.
#cl denotes a cleverness-rating.
#cq denotes a creative quality rating.
#p denotes paperclip.
#g denotes garbage-bag.
#r denotes rope.
#r1 - r7 denote the rater-ids.
#Note that we also relabel the rater-ids for simplicity:
#old rater-id 5 becomes new rater-id 1,
#old rater-id 4 becomes new rater-id 2,
#old rater-id 1 becomes new rater-id 3,
#old rater-id 2 becomes new rater-id 4,
#old rater-id 3 becomes new rater-id 5,
#old rater-id 6 remains     rater-id 6,
#old rater-id 7 remains     rater-id 6.
#The common rater among cleverness and creative quality used to have rater-id 4.
#Now, the rater has rater-id 2:
names(m1) <- c("id",  
               "clpr1", "clpr2", "clpr3", "clpr4", "clpr5",
               "clgr1", "clgr2", "clgr3", "clgr4", "clgr5",
               "clrr1", "clrr2", "clrr3", "clrr4", "clrr5",
               "cqpr2", "cqgr2", "cqrr2",
               "cqpr6", "cqgr6", "cqrr6",
               "cqpr7", "cqgr7", "cqrr7")

#Get all variables to follow the same order of construct, object, and rater
#(clpr1 - clpr5, clgr1-clgr5, clrr1 - clrr5, cqpr2 - cqpr7, cqgr2 - cqgr7, cqrr2 - cqrr7):
m2 <- m1[,c(1:16,17,20,23,18,21,24,19,22,25)]

#This dataset contains the relevant data in a wide-format:
datwide <- m2


#Now we reshape this data into long-format so that it can be used for estimating
#cross-classified and two-level CFAs. There might be more efficient and
#sophisticated ways to do this using specific R-functions but I like to see what
#I am doing at every step.


#Create an id- (target-) and rater-variable for the long-format and bind these
#together in a first (long-format) data-frame:
id <- rep(datwide$id, each = 7)
rater <- rep(c(1:7), 202)
first <- as.data.frame(cbind(id,rater))

#Create separate data frames for each combination of rater
#and AUT-object rated for cleverness:
clpr1dat <- subset(datwide, select = c("id", "clpr1"))
clpr1dat$rater <-1
clpr2dat <- subset(datwide, select = c("id", "clpr2"))
clpr2dat$rater <-2
clpr3dat <- subset(datwide, select = c("id", "clpr3"))
clpr3dat$rater <-3
clpr4dat <- subset(datwide, select = c("id", "clpr4"))
clpr4dat$rater <-4
clpr5dat <- subset(datwide, select = c("id", "clpr5"))
clpr5dat$rater <-5
#
clgr1dat <- subset(datwide, select = c("id", "clgr1"))
clgr1dat$rater <-1
clgr2dat <- subset(datwide, select = c("id", "clgr2"))
clgr2dat$rater <-2
clgr3dat <- subset(datwide, select = c("id", "clgr3"))
clgr3dat$rater <-3
clgr4dat <- subset(datwide, select = c("id", "clgr4"))
clgr4dat$rater <-4
clgr5dat <- subset(datwide, select = c("id", "clgr5"))
clgr5dat$rater <-5
#
clrr1dat <- subset(datwide, select = c("id", "clrr1"))
clrr1dat$rater <-1
clrr2dat <- subset(datwide, select = c("id", "clrr2"))
clrr2dat$rater <-2
clrr3dat <- subset(datwide, select = c("id", "clrr3"))
clrr3dat$rater <-3
clrr4dat <- subset(datwide, select = c("id", "clrr4"))
clrr4dat$rater <-4
clrr5dat <- subset(datwide, select = c("id", "clrr5"))
clrr5dat$rater <-5

#Repeat this process for creative quality ratings:
cqpr2dat <- subset(datwide, select = c("id", "cqpr2"))
cqpr2dat$rater <-2
cqpr6dat <- subset(datwide, select = c("id", "cqpr6"))
cqpr6dat$rater <-6
cqpr7dat <- subset(datwide, select = c("id", "cqpr7"))
cqpr7dat$rater <-7
#
cqgr2dat <- subset(datwide, select = c("id", "cqgr2"))
cqgr2dat$rater <-2
cqgr6dat <- subset(datwide, select = c("id", "cqgr6"))
cqgr6dat$rater <-6
cqgr7dat <- subset(datwide, select = c("id", "cqgr7"))
cqgr7dat$rater <-7
#
cqrr2dat <- subset(datwide, select = c("id", "cqrr2"))
cqrr2dat$rater <-2
cqrr6dat <- subset(datwide, select = c("id", "cqrr6"))
cqrr6dat$rater <-6
cqrr7dat <- subset(datwide, select = c("id", "cqrr7"))
cqrr7dat$rater <-7

#Simplify the names for the score-variables for cleverness 
#and make them equal across all raters:
names(clpr1dat)[2]<-"clp"
names(clpr2dat)[2]<-"clp"
names(clpr3dat)[2]<-"clp"
names(clpr4dat)[2]<-"clp"
names(clpr5dat)[2]<-"clp"
#
names(clgr1dat)[2]<-"clg"
names(clgr2dat)[2]<-"clg"
names(clgr3dat)[2]<-"clg"
names(clgr4dat)[2]<-"clg"
names(clgr5dat)[2]<-"clg"
#
names(clrr1dat)[2]<-"clr"
names(clrr2dat)[2]<-"clr"
names(clrr3dat)[2]<-"clr"
names(clrr4dat)[2]<-"clr"
names(clrr5dat)[2]<-"clr"

#combine all raters for an object rated for cleverness into a single
#data-frame:
clpdat <- rbind(clpr1dat,clpr2dat,clpr3dat,clpr4dat,clpr5dat)
clgdat <- rbind(clgr1dat,clgr2dat,clgr3dat,clgr4dat,clgr5dat)
clrdat <- rbind(clrr1dat,clrr2dat,clrr3dat,clrr4dat,clrr5dat)

#Simplify the names for the score-variables for creative quality 
#and make them equal across all raters:
#
names(cqpr2dat)[2]<-"cqp"
names(cqpr6dat)[2]<-"cqp"
names(cqpr7dat)[2]<-"cqp"
#
names(cqgr2dat)[2]<-"cqg"
names(cqgr6dat)[2]<-"cqg"
names(cqgr7dat)[2]<-"cqg"
#
names(cqrr2dat)[2]<-"cqr"
names(cqrr6dat)[2]<-"cqr"
names(cqrr7dat)[2]<-"cqr"

#combine all raters for an object rated for creative quality into a single
#data-frame:
cqpdat <- rbind(cqpr2dat,cqpr6dat,cqpr7dat)
cqgdat <- rbind(cqgr2dat,cqgr6dat,cqgr7dat)
cqrdat <- rbind(cqrr2dat,cqrr6dat,cqrr7dat)

#Step by step, merge all the long-format data together. We change the order of variables so that 
#first, cleverness-ratings for rope, garbage-bag, and paperclip are given in that order and then,
#creative quality-ratings for rope, garbage-bag, and paperclip are given in that order. This is
#the order as presented in the paper and better fits to the fact that the AUT for rope
#was taken as the standard-indicator for both the first construct of cleverness (j = 1)
#and the second construct of creative quality (j = 2; see, for example, Figure 2):
second <- merge(first,  clrdat, by = c("id", "rater"), all=T)
second <- merge(second, clgdat, by = c("id", "rater"), all=T)
second <- merge(second, clpdat, by = c("id", "rater"), all=T)
second <- merge(second, cqrdat, by = c("id", "rater"), all=T)
second <- merge(second, cqgdat, by = c("id", "rater"), all=T)
second <- merge(second, cqpdat, by = c("id", "rater"), all=T)

#This gives us, finally, our complete data-set in long-format for
#estimating cross-classified (cc) CFAs (and two-level CFAs):
ccdat <- second

#Make a separate data-set for Mplus in which NAs (missing values) are replaced by
#999 which is not a possible score (this is common practice in Mplus):
ccdat_mplus <- ccdat
ccdat_mplus[is.na(ccdat_mplus)] <- 999

#For our analyses, we exclude the cleverness-ratings of rater 2
#(see paper for details). Thus, these ratings are re-declared as
#missings (999 for Mplus). We save this data-frame with additional missings
#as a new one called "ccdat_mplus_use":
ccdat_mplus_use <- ccdat_mplus
for (i in 1:nrow(ccdat_mplus_use)){
  if (ccdat_mplus_use$rater[i] == 2){
    ccdat_mplus_use$clr[i] <- 999
    ccdat_mplus_use$clg[i] <- 999
    ccdat_mplus_use$clp[i] <- 999
  }
}

#Replace the elaborate target-ids with simple target-numbers:
ccdat_mplus_use$id <- rep(c(1:202), each = 7)

#Rename the variables so that they are the same as in the paper (e. g., Figure 2):
names(ccdat_mplus_use) <- c("target", "rater", "y11", "y21", "y31", "y12", "y22", "y32")

#Finally, save the data as a .dat-file for analyses with Mplus
#(remove the ### to run the code):
   ### write.table(ccdat_mplus_use, file = "ccdat_mplus_use.dat", col.names = F, row.names=F)

